// Generated by CoffeeScript 1.12.4
(function() {
  var ApiClient, EventEmitter, P, PushWebSocket, WebSocket, _, auth, config, errors, isNode, jsonify, logger, makeRecord, moment, request,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  _ = require('lodash');

  P = require('bluebird');

  moment = require('moment');

  request = require('request');

  EventEmitter = require('eventemitter3');

  auth = require('./auth');

  config = require('./config');

  errors = require('./errors');

  logger = require('./logger');

  WebSocket = require('./ws')();

  jsonify = function(obj) {
    return JSON.stringify(obj, null, 2);
  };

  makeRecord = function(cfg) {
    var record;
    return record = {
      access_key: cfg.api_key.access,
      client_salt: cfg.client_key.salt,
      timestamp: moment.utc().toISOString()
    };
  };

  isNode = function() {
    return window === void 0;
  };

  PushWebSocket = (function(superClass) {
    extend(PushWebSocket, superClass);

    function PushWebSocket(cfg1, namespace1, attributes1, autoAcknowledge1) {
      var ref;
      this.cfg = cfg1;
      this.namespace = namespace1;
      this.attributes = attributes1;
      this.autoAcknowledge = autoAcknowledge1 != null ? autoAcknowledge1 : true;
      PushWebSocket.__super__.constructor.call(this);
      this.baseWsUrl = this.cfg.base_ws_url;
      this.sock = null;
      this.pingerRef = null;
      this.messageCount = 0;
      this.lastMessageId = null;
      this.autoReconnect = (ref = this.cfg.websocket_auto_reconnect) != null ? ref : true;
    }

    PushWebSocket.prototype.close = function() {
      var error;
      this.autoReconnect = false;
      if (this.pingerRef != null) {
        try {
          clearInterval(this.pingerRef);
        } catch (error1) {
          error = error1;
          logger.error("Error clearing ping interval: " + error + "\n" + error.stack);
        } finally {
          this.pingerRef = null;
        }
      }
      if (this.sock != null) {
        try {
          return this.sock.close();
        } catch (error1) {
          error = error1;
          return logger.error("Error while closing WebSocket: " + error + "\n" + error.stack);
        } finally {
          this.sock = null;
        }
      }
    };

    PushWebSocket.prototype.disconnect = function() {
      return this.close();
    };

    PushWebSocket.prototype.ack = function(messageId) {
      var acknowledgement;
      if (this.sock != null) {
        acknowledgement = {
          event: "message-received",
          message_id: messageId
        };
        logger.verbose("Acknowledging message " + messageId + " with payload " + acknowledgement);
        return this.sock.send(JSON.stringify(acknowledgement)).then((function(_this) {
          return function() {
            return _this.emit('acked', messageId);
          };
        })(this))["catch"](function(error) {
          return logger.error("Error sending acknowledgement for message '" + messageId + "': " + error + "\n" + error.stack);
        });
      } else {
        return logger.warn("The connection is not open, therefore message '" + messageId + "' cannot be acknowledged.");
      }
    };

    PushWebSocket.prototype.acknowledge = function(messageId) {
      return this.ack(messageId);
    };

    PushWebSocket.prototype.connect = function() {
      var connectHandler;
      connectHandler = (function(_this) {
        return function(resolve, reject) {
          var data, error, hasConnected, headers, record, timeout, url;
          if (_this.sock != null) {
            return resolve();
          } else {
            record = makeRecord(_this.cfg);
            record.namespace = _this.namespace;
            record.attributes = _this.attributes;
            data = auth.signRecord(_this.cfg.client_key.secret, record);
            hasConnected = false;
            url = _this.baseWsUrl + "/push";
            headers = {
              'Payload-HMAC': data.hmac,
              'JSON-Base64': data.bufferB64
            };
            timeout = _this.cfg.websocket_connect_timeout;
            try {
              _this.sock = new WebSocket(url, headers, timeout);
              _this.sock.once('close', function() {
                var error, message, reconnect;
                if (_this.autoReconnect === true) {
                  _this.sock = null;
                  if (_this.pingerRef != null) {
                    try {
                      clearInterval(_this.pingerRef);
                    } catch (error1) {
                      error = error1;
                      logger.error("Error clearing pinger interval: " + error + "\n" + error.stack);
                    } finally {
                      _this.pingerRef = null;
                    }
                  }
                  reconnect = function() {
                    return _this.connect().then(function() {
                      logger.info("Push WebSocket replaced for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)));
                      return _this.emit('reconnect');
                    })["catch"](function(error) {
                      logger.error("Error replacing push WebSocket for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)) + " : " + error + "\n" + error.stack);
                      return _this.emit('error', error);
                    });
                  };
                  logger.info("Connection closed. Reconnecting in 5 seconds.");
                  return setTimeout(reconnect, 5000);
                } else {
                  _this.emit('close');
                  if (hasConnected) {
                    return logger.info("Push WebSocket closed for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)));
                  } else {
                    message = "Websocket closed before the connection was established for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes));
                    logger.info(message);
                    return reject(new Error(message));
                  }
                }
              });
              _this.sock.once('open', function() {
                var pinger;
                logger.info("Push WebSocket opened for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)));
                hasConnected = true;
                _this.emit('open');
                pinger = function() {
                  logger.verbose("Sending PING to keep the push WebSocket alive.");
                  if (_this.sock != null) {
                    return _this.sock.ping();
                  }
                };
                _this.pingerRef = setInterval(pinger, 15000);
                return resolve();
              });
              _this.sock.on('error', function(error) {
                _this.emit('error', error);
                if (!hasConnected) {
                  logger.error("WebSocket connect error for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)) + " : " + error + "\n" + error.stack);
                  return reject(error);
                } else {
                  return logger.error("WebSocket error for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)) + " : " + error + "\n" + error.stack);
                }
              });
              _this.sock.on('message', function(msg) {
                var error, message;
                logger.verbose("Received message from namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)) + " :", msg);
                try {
                  _this.messageCount += 1;
                  message = JSON.parse(msg);
                  _this.emit('message', message);
                  _this.lastMessageId = message.message_id;
                  if (_this.autoAcknowledge === true) {
                    return _this.ack(message.message_id);
                  }
                } catch (error1) {
                  error = error1;
                  return logger.error("Invalid push message received: " + error + "\n" + error.stack);
                }
              });
              return _this.sock.once('connectFailed', function(error) {
                _this.emit('connectFailed', error);
                logger.error("Failed to connect to push WebSocket for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)));
                return reject(new errors.ApiError("Server rejected the push WebSocket", error));
              });
            } catch (error1) {
              error = error1;
              logger.error("Error creating WebSocket for namespace '" + _this.namespace + "' channel " + (jsonify(_this.attributes)));
              return reject(new errors.ApiError("Error creating the push WebSocket", error));
            }
          }
        };
      })(this);
      return new P(connectHandler);
    };

    return PushWebSocket;

  })(EventEmitter);

  ApiClient = (function() {
    function ApiClient(cfg1) {
      this.cfg = cfg1;
      this._initialized = true;
    }

    ApiClient.prototype.baseUrl = function() {
      var ref, ref1;
      return (ref = (ref1 = this.cfg) != null ? ref1.base_url : void 0) != null ? ref : void 0;
    };

    ApiClient.prototype.baseWsUrl = function() {
      var ref, ref1;
      return (ref = (ref1 = this.cfg) != null ? ref1.base_ws_url : void 0) != null ? ref : void 0;
    };

    ApiClient.prototype.accessKey = function() {
      var ref, ref1, ref2;
      return (ref = (ref1 = this.cfg) != null ? (ref2 = ref1.api_key) != null ? ref2.access : void 0 : void 0) != null ? ref : void 0;
    };

    ApiClient.prototype.clientSalt = function() {
      var ref, ref1, ref2;
      return (ref = (ref1 = this.cfg) != null ? (ref2 = ref1.client_key) != null ? ref2.salt : void 0 : void 0) != null ? ref : void 0;
    };

    ApiClient.prototype.clientSecret = function() {
      var ref, ref1, ref2;
      return (ref = (ref1 = this.cfg) != null ? (ref2 = ref1.client_key) != null ? ref2.secret : void 0 : void 0) != null ? ref : void 0;
    };

    ApiClient.prototype.subscribe = function(namespace, attributes, autoAcknowledge) {
      var cause, error, ws;
      if (autoAcknowledge == null) {
        autoAcknowledge = true;
      }
      try {
        ws = new PushWebSocket(this.cfg, namespace, attributes, autoAcknowledge);
        ws.connect();
        return ws;
      } catch (error1) {
        cause = error1;
        error = new PushError("Error creating the subscription WebSocket '" + namespace + "' channel " + (jsonify(attributes)), cause);
        logger.error(error);
        throw error;
      }
    };

    ApiClient.prototype.sendEvent = function(namespace, eventName, attributes, options) {
      var data, record, ref, ref1, ref2, ref3, ref4;
      if (options == null) {
        options = void 0;
      }
      record = makeRecord(this.cfg);
      record.namespace = namespace;
      record.event_name = eventName;
      record.attributes = attributes;
      record.tags = (ref = options != null ? options.tags : void 0) != null ? ref : void 0;
      record.debug_directive = (ref1 = options != null ? options.debugDirective : void 0) != null ? ref1 : void 0;
      record.tracing = (ref2 = options != null ? options.eventTracing : void 0) != null ? ref2 : false;
      record.trace_key = (ref3 = (ref4 = options != null ? options.trace_key : void 0) != null ? ref4 : this.cfg.event_trace_key) != null ? ref3 : void 0;
      data = auth.signRecord(this.cfg.client_key.secret, record);
      return this.makeRequest('POST', "/event", data);
    };

    ApiClient.prototype.getEventTrace = function(namespace, attributes, eventId) {
      var data, record;
      record = makeRecord(this.cfg);
      record.namespace = namespace;
      record.attributes = attributes;
      data = auth.signRecord(this.cfg.client_key.secret, record);
      return this.makeRequest('GET', "/event_trace/" + eventId, data);
    };

    ApiClient.prototype.getChannelSummary = function(namespace, attributes) {
      var data, record;
      record = makeRecord(this.cfg);
      record.namespace = namespace;
      record.attributes = attributes;
      data = auth.signRecord(this.cfg.client_key.secret, record);
      return this.makeRequest('POST', "/channel_summary", data);
    };

    ApiClient.prototype.getMessage = function(namespace, attributes, messageId) {
      var data, record;
      record = makeRecord(this.cfg);
      record.namespace = namespace;
      record.attributes = attributes;
      data = auth.signRecord(this.cfg.client_key.secret, record);
      return this.makeRequest('GET', "/message/" + messageId, data);
    };

    ApiClient.prototype.makeRequest = function(method, path, data) {
      return new P((function(_this) {
        return function(resolve, reject) {
          var contentType, isGet, jsonB64Header, options, payload;
          isGet = method === 'GET';
          contentType = !isGet ? 'application/json' : void 0;
          jsonB64Header = isGet ? data.bufferB64 : void 0;
          payload = !isGet ? data.buffer : void 0;
          options = {
            uri: "" + (_this.baseUrl()) + path,
            method: method,
            headers: {
              'Payload-HMAC': data.hmac,
              'Content-Type': contentType,
              'JSON-Base64': jsonB64Header
            },
            body: payload,
            timeout: _this.cfg.http_request_timeout
          };
          return request(options, function(error, response) {
            var json, record;
            if (error != null) {
              return reject(new errors.ApiError("Error attempting to send a request to the Cogs server", error));
            } else if (response.statusCode !== 200) {
              try {
                record = JSON.parse(json);
                json = JSON.stringify(record, null, 2);
                return reject(new errors.ApiError("Received an error response from the server", void 0, response.statusCode, json));
              } catch (error1) {
                error = error1;
                return reject(new errors.ApiError("Received an error response from the server", void 0, response.statusCode, response.body));
              }
            } else {
              try {
                return resolve(JSON.parse(response.body));
              } catch (error1) {
                error = error1;
                return reject(new errors.ApiError("Error parsing response body (expected valid JSON)", error));
              }
            }
          });
        };
      })(this));
    };

    return ApiClient;

  })();

  module.exports = {
    getClient: function(configPath) {
      return config.getConfig(configPath).then(function(cfg) {
        if (cfg.log_level != null) {
          logger.setLogLevel(cfg.log_level);
        }
        return new ApiClient(cfg);
      });
    },
    getClientWithConfig: function(cfg) {
      return config.validateConfig(cfg).then(function(cfg) {
        if (cfg.log_level != null) {
          logger.setLogLevel(cfg.log_level);
        }
        return new ApiClient(cfg);
      });
    }
  };

}).call(this);
