// Generated by CoffeeScript 1.12.4
(function() {
  var KEY_PARTS, _, crypto, errors, moment, signRecord, socketAuth, splitKey, xor;

  _ = require('lodash');

  crypto = require('crypto');

  moment = require('moment');

  xor = require('buffer-xor');

  errors = require('./errors');

  KEY_PARTS = ['R', 'W', 'A'];

  splitKey = function(key) {
    var identity, parts, perm, permKey;
    parts = key.split('-');
    if (parts.length !== 3) {
      throw new errors.AuthKeyError("Invalid format for project key.", void 0);
    }
    perm = parts[0], identity = parts[1], permKey = parts[2];
    if (!KEY_PARTS.includes(perm)) {
      throw new errors.AuthKeyError("Invalid permission prefix for project key.", void 0);
    }
    if (!identity.match(/^[0-9a-fA-F]+$/)) {
      throw new errors.AuthKeyError("Invalid format for key identity.", void 0);
    }
    if (!permKey.match(/^[0-9a-fA-F]+$/)) {
      throw new errors.AuthKeyError("Invalid format for perm key.", void 0);
    }
    return {
      perm: perm,
      identity: identity,
      key: permKey
    };
  };

  socketAuth = function(keys, sessionUuid) {
    var auth, hmac, identity, keyObjs, perms, record, sigs;
    if (keys.length < 1) {
      throw errors.AuthKeyException("No keys supplied.", void 0);
    }
    keys = _.uniqBy(keys, function(key) {
      return key.substr(0, 1);
    });
    keyObjs = keys.map(function(key) {
      return splitKey(key);
    });
    perms = keyObjs.map(function(arg) {
      var perm;
      perm = arg.perm;
      return perm;
    }).join('');
    identity = _(keyObjs).map(function(arg) {
      var identity;
      identity = arg.identity;
      return identity;
    }).first();
    record = {
      identity: identity,
      permissions: perms,
      security_timestamp: moment.utc().toISOString(),
      session_uuid: sessionUuid
    };
    sigs = _(keyObjs).map(function(arg) {
      var key;
      key = arg.key;
      return signRecord(key, record);
    }).value();
    hmac = _(sigs).map(function(arg) {
      var hmac;
      hmac = arg.hmac;
      return new Buffer(hmac, 'hex');
    }).reduce(xor, Buffer.alloc(32));
    auth = _(sigs).first();
    auth.hmac = hmac.toString('hex');
    auth.json = auth.buffer.toString();
    auth.permissions = record.permissions;
    return auth;
  };

  signRecord = function(hexKey, record) {
    var hmac, hmacHexDigest, json, jsonBuffer, jsonBufferB64, keyBuffer, signatureSummary;
    json = JSON.stringify(record);
    keyBuffer = new Buffer(hexKey, 'hex');
    jsonBuffer = new Buffer(json, 'utf-8');
    jsonBufferB64 = jsonBuffer.toString('base64');
    hmac = crypto.createHmac('SHA256', keyBuffer);
    hmac.update(jsonBuffer);
    hmacHexDigest = hmac.digest('hex');
    signatureSummary = {
      hmac: hmacHexDigest,
      buffer: jsonBuffer,
      bufferB64: jsonBufferB64,
      contentLength: jsonBuffer.length,
      record: record
    };
    return signatureSummary;
  };

  module.exports = {
    signRecord: signRecord,
    socketAuth: socketAuth
  };

}).call(this);
