// Generated by CoffeeScript 1.12.4
(function() {
  var Joi, P, errors, fs, getConfig, logger, parseConfig, readConfig, readJson, validateConfig, validateWithJoi;

  P = require('bluebird');

  Joi = require('joi');

  fs = P.promisifyAll(require('fs'));

  logger = require('./logger');

  errors = require('./errors');

  validateWithJoi = function(config) {
    var schema;
    schema = Joi.object().keys({
      base_url: Joi.string().uri().optional()["default"]("https://api.cogswell.io", "URL for the Cogswell API."),
      base_ws_url: Joi.string().uri().optional(),
      api_key: Joi.object().keys({
        access: Joi.string().regex(/^[0-9a-fA-F]{32}$/).required(),
        secret: Joi.string().regex(/^[0-9a-fA-F]{128}$/).optional()
      }).required(),
      client_key: Joi.object().keys({
        salt: Joi.string().regex(/^[0-9a-fA-F]{64}$/).required(),
        secret: Joi.string().regex(/^[0-9a-fA-F]{64}$/).required()
      }).optional(),
      http_request_timeout: Joi.number().integer().min(250).max(900000).optional()["default"](30000, 'Default timeout of 30,000 milliseconds for HTTP requests.'),
      websocket_connect_timeout: Joi.number().integer().min(250).max(900000).optional()["default"](30000, 'Default timeout of 30,000 milliseconds for WebSocket connects.'),
      websocket_auto_reconnect: Joi.boolean().optional()["default"](true, 'Use this field to control auto-reconnect, defaults to true.'),
      log_level: Joi.only('off', 'error', 'warn', 'info', 'verbose', 'debug').optional()["default"]('error', 'Sets the log level for the SDK, defaults to "error"'),
      event_trace_key: Joi.string().min(1).optional('Auth key required in order to enable event tracing (intended for internal use only)')
    });
    return P.promisify(Joi.validate)(config, schema);
  };

  validateConfig = function(configObj) {
    return new P(function(resolve, reject) {
      var config;
      return config = validateWithJoi(configObj).then(function(config) {
        if (config.base_ws_url == null) {
          config.base_ws_url = config.base_url.replace(/http/, 'ws');
        }
        return resolve(config);
      })["catch"](function(error) {
        var err;
        err = new errors.ConfigError("Error validating the config", error);
        logger.error(err);
        return reject(err);
      });
    });
  };

  parseConfig = function(configJson) {
    return new P(function(resolve, reject) {
      var configObj, err, error;
      try {
        configObj = JSON.parse(configJson);
        return resolve(validateConfig(configObj));
      } catch (error1) {
        error = error1;
        logger.error(err);
        err = new errors.ConfigError("Error parsing config JSON", error);
        return reject(err);
      }
    });
  };

  readConfig = function(configPath) {
    return fs.readFileAsync(configPath).then(function(configJson) {
      return parseConfig(configJson);
    });
  };

  getConfig = function(configPath) {
    return new P(function(resolve, reject) {
      return fs.exists(configPath, function(exists) {
        if (!exists) {
          return reject(new Error("Config file '" + configPath + "' not found"));
        } else {
          return resolve(readConfig(configPath));
        }
      });
    });
  };

  readJson = function(path) {
    return fs.readFileAsync(path).then(function(raw) {
      return P["try"](JSON.parse(raw));
    });
  };

  module.exports = {
    getConfig: getConfig,
    readJson: readJson,
    validateConfig: validateConfig
  };

}).call(this);
