// Generated by CoffeeScript 1.12.4
(function() {
  var P, ToolsClient, _, api, auth, config, errors, logger, makeRecord, makeSigned, moment, request;

  _ = require('lodash');

  P = require('bluebird');

  moment = require('moment');

  request = require('request');

  api = require('./api');

  auth = require('./auth');

  config = require('./config');

  errors = require('./errors');

  logger = require('./logger');

  makeRecord = function(cfg) {
    var record;
    return record = {
      access_key: cfg.api_key.access,
      timestamp: moment.utc().toISOString()
    };
  };

  makeSigned = function(cfg) {
    var record;
    record = makeRecord(cfg);
    return auth.signRecord(cfg.api_key.secret, record);
  };

  ToolsClient = (function() {
    function ToolsClient(cfg1) {
      this.cfg = cfg1;
      this._initialized = true;
    }

    ToolsClient.prototype.baseUrl = function() {
      var ref, ref1;
      return (ref = (ref1 = this.cfg) != null ? ref1.base_url : void 0) != null ? ref : void 0;
    };

    ToolsClient.prototype.baseWsUrl = function() {
      var ref, ref1;
      return (ref = (ref1 = this.cfg) != null ? ref1.base_ws_url : void 0) != null ? ref : void 0;
    };

    ToolsClient.prototype.accessKey = function() {
      var ref, ref1, ref2;
      return (ref = (ref1 = this.cfg) != null ? (ref2 = ref1.api_key) != null ? ref2.access : void 0 : void 0) != null ? ref : void 0;
    };

    ToolsClient.prototype.secretKey = function() {
      var ref, ref1, ref2;
      return (ref = (ref1 = this.cfg) != null ? (ref2 = ref1.api_key) != null ? ref2.secret : void 0 : void 0) != null ? ref : void 0;
    };

    ToolsClient.prototype.getApiClientWithNewKey = function() {
      return this.newClientKey().then((function(_this) {
        return function(record) {
          var apiCfg;
          apiCfg = _.cloneDeep(_this.cfg);
          apiCfg.api_key.secret = void 0;
          apiCfg.client_key = {
            salt: record.client_salt,
            secret: record.client_secret
          };
          return api.getClientWithConfig(apiCfg);
        };
      })(this));
    };

    ToolsClient.prototype.getNamespaceSchema = function(namespace) {
      var data;
      data = makeSigned(this.cfg);
      return this.makeRequest('GET', "/namespace/" + namespace + "/schema", data);
    };

    ToolsClient.prototype.newRandomUuid = function() {
      var data;
      data = makeSigned(this.cfg);
      return this.makeRequest('POST', "/random_uuid", data);
    };

    ToolsClient.prototype.newClientKey = function() {
      var data;
      data = makeSigned(this.cfg);
      return this.makeRequest('POST', "/client_secret", data);
    };

    ToolsClient.prototype.makeRequest = function(method, path, data) {
      return new P((function(_this) {
        return function(resolve, reject) {
          var contentType, isGet, jsonB64Header, options, payload, url;
          isGet = method === 'GET';
          contentType = !isGet ? 'application/json' : void 0;
          jsonB64Header = isGet ? data.bufferB64 : void 0;
          payload = !isGet ? data.buffer : void 0;
          url = "" + (_this.baseUrl()) + path;
          options = {
            uri: url,
            method: method,
            headers: {
              'Payload-HMAC': data.hmac,
              'Content-Type': contentType,
              'JSON-Base64': jsonB64Header
            },
            body: payload,
            timeout: _this.cfg.http_request_timeout
          };
          return request(options, function(error, response) {
            var json, record;
            if (error != null) {
              return reject(new errors.ToolsError("Error attempting to send a request to the Cogs server", error));
            } else if (response.statusCode !== 200) {
              try {
                record = JSON.parse(json);
                json = JSON.stringify(record, null, 2);
                return reject(new errors.ToolsError("Received an error response from the server", void 0, response.statusCode, json));
              } catch (error1) {
                error = error1;
                return reject(new errors.ToolsError("Received an error response from the server", void 0, response.statusCode, response.body));
              }
            } else {
              try {
                return resolve(JSON.parse(response.body));
              } catch (error1) {
                error = error1;
                return reject(new errors.ToolsError("Error parsing response body (expected valid JSON)", error));
              }
            }
          });
        };
      })(this));
    };

    return ToolsClient;

  })();

  module.exports = {
    getClient: function(configPath) {
      return config.getConfig(configPath).then(function(cfg) {
        if (cfg.log_level != null) {
          logger.setLogLevel(cfg.log_level);
        }
        return new ToolsClient(cfg);
      });
    },
    getClientWithConfig: function(cfg) {
      if (cfg.log_level != null) {
        logger.setLogLevel(cfg.log_level);
      }
      return P.resolve(new ToolsClient(cfg));
    }
  };

}).call(this);
